from __future__ import unicode_literals
import csv as py_csv
try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO
from functools import wraps


def csv_serializer(field_names=None, header=True):
    def inner_function(function):
        @wraps(function)
        def wrapper(*args, **kwargs):
            rows = function(*args, **kwargs)
            csv_string = StringIO()
            fields = field_names
            if fields is None:
                if rows:
                    fields = sorted(rows[0].keys())
            writer = py_csv.DictWriter(csv_string, fieldnames=fields, extrasaction='ignore')
            if header and fields:
                writer.writeheader()
            writer.writerows(rows)
            return csv_string.getvalue()
        return wrapper
    return inner_function



from functools import wraps


def length(exact=None, minimum=None, maximum=None):
    def inner_function(function):
        @wraps(function)
        def wrapper(*args, **kwargs):
            value = function(*args, **kwargs)
            value_length = len(value)
            assert any([exact, minimum, maximum]), "Must define one or more of exact, minimum, maximum kwargs"
            if exact is not None:
                assert minimum is None and maximum is None, "Cannot define both exact and minimum/maximum"
                assert value_length == exact, "Invalid length %s != %s" % (value_length, exact)
            else:
                if minimum:
                    assert value_length >= minimum, "Invalid length %s < %s" % (value_length, minimum)
                if maximum:
                    assert value_length <= maximum, "Invalid length %s > %s" % (value_length, maximum)
            return value
        return wrapper
    return inner_function


import string
from functools import wraps


def _filter_chars(data, allowed):
    return "".join(filter(lambda x: x in allowed, data))


def printable():
    def inner_function(function):
        @wraps(function)
        def wrapper(*args, **kwargs):
            return _filter_chars(function(*args, **kwargs), string.printable)
        return wrapper
    return inner_function


def ascii():
    def inner_function(function):
        @wraps(function)
        def wrapper(*args, **kwargs):
            return _filter_chars(function(*args, **kwargs), string.ascii_letters)
        return wrapper
    return inner_function


def ascii_lower():
    def inner_function(function):
        @wraps(function)
        def wrapper(*args, **kwargs):
            return _filter_chars(function(*args, **kwargs), string.ascii_lowercase)
        return wrapper
    return inner_function


def ascii_upper():
    def inner_function(function):
        @wraps(function)
        def wrapper(*args, **kwargs):
            return _filter_chars(function(*args, **kwargs), string.ascii_uppercase)
        return wrapper
    return inner_function


def digits():
    def inner_function(function):
        @wraps(function)
        def wrapper(*args, **kwargs):
            return _filter_chars(function(*args, **kwargs), string.digits)
        return wrapper
    return inner_function
